@@ -21,6 +21,7 @@
  * code by Owen Taylor <otaylor@redhat.com> and 
  * Carsten Haitzler <raster@rasterman.com>
  */
+#include <config.h>
 
 #include "theme-pixbuf.h"
 #include "sapwood-style.h"
@@ -50,6 +51,7 @@ theme_symbols[] =
   { "file", 		TOKEN_FILE },
   { "stretch", 		TOKEN_STRETCH },
   { "border", 		TOKEN_BORDER },
+  { "shaped",           TOKEN_SHAPED },
   { "detail", 		TOKEN_DETAIL },
   { "state", 		TOKEN_STATE },
   { "shadow", 		TOKEN_SHADOW },
@@ -497,6 +499,34 @@ theme_parse_orientation(GScanner * scanner,
   return G_TOKEN_NONE;
 }
 
+static guint
+theme_parse_shaped(GScanner * scanner,
+		   ThemeImage * data)
+{
+  guint    token;
+  gboolean shaped;
+
+  token = g_scanner_get_next_token(scanner);
+  if (token != TOKEN_SHAPED)
+    return TOKEN_SHAPED;
+
+  token = g_scanner_get_next_token(scanner);
+  if (token != G_TOKEN_EQUAL_SIGN)
+    return G_TOKEN_EQUAL_SIGN;
+
+  token = g_scanner_get_next_token(scanner);
+  if (token == TOKEN_TRUE)
+    shaped = TRUE;
+  else if (token == TOKEN_FALSE)
+    shaped = FALSE;
+  else
+    return TOKEN_TRUE;
+
+  data->background_shaped = shaped != FALSE;
+
+  return G_TOKEN_NONE;
+}
+
 static void
 theme_image_ref (ThemeImage *data)
 {
@@ -543,20 +573,9 @@ theme_parse_image(GtkSettings  *settings,
   if (token != G_TOKEN_LEFT_CURLY)
     return G_TOKEN_LEFT_CURLY;
 
-  data = g_malloc(sizeof(ThemeImage));
-
+  data = g_new0 (ThemeImage, 1);
   data->refcount = 1;
 
-  data->background = NULL;
-  data->overlay = NULL;
-  data->gap_start = NULL;
-  data->gap = NULL;
-  data->gap_end = NULL;
-
-  data->match_data.function = 0;
-  data->match_data.detail = NULL;
-  data->match_data.flags = 0;
-
   token = g_scanner_peek_next_token(scanner);
   while (token != G_TOKEN_RIGHT_CURLY)
     {
@@ -592,6 +611,9 @@ theme_parse_image(GtkSettings  *settings,
 	case TOKEN_STRETCH:
 	  token = theme_parse_stretch(scanner, &data->background);
 	  break;
+	case TOKEN_SHAPED:
+	  token = theme_parse_shaped(scanner, data);
+	  break;
 	case TOKEN_GAP_FILE:
 	  token = theme_parse_file(settings, scanner, &data->gap);
 	  break;
